@@ -204,8 +204,7 @@ public class WarWatcher {
 
         @Override
         public boolean equals(Object other) {
-            if (other instanceof WarInfo) {
-                WarInfo wo = (WarInfo) other;
+            if (other instanceof WarInfo wo) {
                 return wo.getWar().equals(getWar());
             } else {
                 return false;
