@@ -52,10 +52,8 @@ public class MBeanUtils {
      * The set of exceptions to the normal rules used by <code>createManagedBean()</code>. The first element of each
      * pair is a class name, and the second element is the managed bean name.
      */
-    private static final String[][] exceptions = { { "org.apache.catalina.users.MemoryGroup", "Group" },
-            { "org.apache.catalina.users.MemoryRole", "Role" }, { "org.apache.catalina.users.MemoryUser", "User" },
-            { "org.apache.catalina.users.GenericGroup", "Group" }, { "org.apache.catalina.users.GenericRole", "Role" },
-            { "org.apache.catalina.users.GenericUser", "User" } };
+    private static final String[][] exceptions = { { "org.apache.catalina.users.GenericGroup", "Group" },
+            { "org.apache.catalina.users.GenericRole", "Role" }, { "org.apache.catalina.users.GenericUser", "User" } };
 
 
     /**
