@@ -17,6 +17,7 @@
 package org.apache.catalina.tribes.tipis;
 
 import java.io.IOException;
+import java.io.Serial;
 import java.io.Serializable;
 import java.nio.charset.StandardCharsets;
 import java.util.ArrayList;
@@ -55,6 +56,7 @@ import org.apache.juli.logging.LogFactory;
 public abstract class AbstractReplicatedMap<K, V>
         implements Map<K,V>, Serializable, RpcCallback, ChannelListener, MembershipListener, Heartbeat {
 
+    @Serial
     private static final long serialVersionUID = 1L;
 
     protected static final StringManager sm = StringManager.getManager(AbstractReplicatedMap.class);
@@ -552,10 +554,9 @@ public abstract class AbstractReplicatedMap<K, V>
 
     @Override
     public Serializable replyRequest(Serializable msg, final Member sender) {
-        if (!(msg instanceof MapMessage)) {
+        if (!(msg instanceof MapMessage mapmsg)) {
             return null;
         }
-        MapMessage mapmsg = (MapMessage) msg;
 
         // map init request
         if (mapmsg.getMsgType() == MapMessage.MSG_INIT) {
@@ -615,11 +616,10 @@ public abstract class AbstractReplicatedMap<K, V>
     @Override
     public void leftOver(Serializable msg, Member sender) {
         // left over membership messages
-        if (!(msg instanceof MapMessage)) {
+        if (!(msg instanceof MapMessage mapmsg)) {
             return;
         }
 
-        MapMessage mapmsg = (MapMessage) msg;
         try {
             mapmsg.deserialize(getExternalLoaders());
             if (mapmsg.getMsgType() == MapMessage.MSG_START) {
@@ -649,11 +649,10 @@ public abstract class AbstractReplicatedMap<K, V>
     @SuppressWarnings("unchecked")
     @Override
     public void messageReceived(Serializable msg, Member sender) {
-        if (!(msg instanceof MapMessage)) {
+        if (!(msg instanceof MapMessage mapmsg)) {
             return;
         }
 
-        MapMessage mapmsg = (MapMessage) msg;
         if (log.isTraceEnabled()) {
             log.trace("Map[" + mapname + "] received message:" + mapmsg);
         }
@@ -713,8 +712,7 @@ public abstract class AbstractReplicatedMap<K, V>
                 entry.setCopy(mapmsg.getMsgType() == MapMessage.MSG_COPY);
                 entry.setBackupNodes(mapmsg.getBackupNodes());
                 entry.setPrimary(mapmsg.getPrimary());
-                if (entry.getValue() instanceof ReplicatedMapEntry) {
-                    ReplicatedMapEntry diff = (ReplicatedMapEntry) entry.getValue();
+                if (entry.getValue() instanceof ReplicatedMapEntry diff) {
                     if (mapmsg.isDiff()) {
                         diff.lock();
                         try {
@@ -726,8 +724,7 @@ public abstract class AbstractReplicatedMap<K, V>
                         }
                     } else {
                         if (mapmsg.getValue() != null) {
-                            if (mapmsg.getValue() instanceof ReplicatedMapEntry) {
-                                ReplicatedMapEntry re = (ReplicatedMapEntry) mapmsg.getValue();
+                            if (mapmsg.getValue() instanceof ReplicatedMapEntry re) {
                                 re.setOwner(getMapOwner());
                                 entry.setValue((V) re);
                             } else {
@@ -737,8 +734,7 @@ public abstract class AbstractReplicatedMap<K, V>
                             ((ReplicatedMapEntry) entry.getValue()).setOwner(getMapOwner());
                         }
                     } // end if
-                } else if (mapmsg.getValue() instanceof ReplicatedMapEntry) {
-                    ReplicatedMapEntry re = (ReplicatedMapEntry) mapmsg.getValue();
+                } else if (mapmsg.getValue() instanceof ReplicatedMapEntry re) {
                     re.setOwner(getMapOwner());
                     entry.setValue((V) re);
                 } else {
@@ -1069,8 +1065,7 @@ public abstract class AbstractReplicatedMap<K, V>
                     if (dest != null && dest.length > 0) {
                         getChannel().send(dest, msg, getChannelSendOptions());
                     }
-                    if (entry.getValue() instanceof ReplicatedMapEntry) {
-                        ReplicatedMapEntry val = (ReplicatedMapEntry) entry.getValue();
+                    if (entry.getValue() instanceof ReplicatedMapEntry val) {
                         val.setOwner(getMapOwner());
                     }
                 } else if (entry.isCopy()) {
@@ -1446,6 +1441,7 @@ public abstract class AbstractReplicatedMap<K, V>
     // ------------------------------------------------------------------------------
 
     public static class MapMessage implements Serializable, Cloneable {
+        @Serial
         private static final long serialVersionUID = 1L;
         public static final int MSG_BACKUP = 1;
         public static final int MSG_RETRIEVE_BACKUP = 2;
@@ -1479,36 +1475,22 @@ public abstract class AbstractReplicatedMap<K, V>
         }
 
         public String getTypeDesc() {
-            switch (msgtype) {
-                case MSG_BACKUP:
-                    return "MSG_BACKUP";
-                case MSG_RETRIEVE_BACKUP:
-                    return "MSG_RETRIEVE_BACKUP";
-                case MSG_PROXY:
-                    return "MSG_PROXY";
-                case MSG_REMOVE:
-                    return "MSG_REMOVE";
-                case MSG_STATE:
-                    return "MSG_STATE";
-                case MSG_START:
-                    return "MSG_START";
-                case MSG_STOP:
-                    return "MSG_STOP";
-                case MSG_INIT:
-                    return "MSG_INIT";
-                case MSG_STATE_COPY:
-                    return "MSG_STATE_COPY";
-                case MSG_COPY:
-                    return "MSG_COPY";
-                case MSG_ACCESS:
-                    return "MSG_ACCESS";
-                case MSG_NOTIFY_MAPMEMBER:
-                    return "MSG_NOTIFY_MAPMEMBER";
-                case MSG_PING:
-                    return "MSG_PING";
-                default:
-                    return "UNKNOWN";
-            }
+            return switch (msgtype) {
+                case MSG_BACKUP -> "MSG_BACKUP";
+                case MSG_RETRIEVE_BACKUP -> "MSG_RETRIEVE_BACKUP";
+                case MSG_PROXY -> "MSG_PROXY";
+                case MSG_REMOVE -> "MSG_REMOVE";
+                case MSG_STATE -> "MSG_STATE";
+                case MSG_START -> "MSG_START";
+                case MSG_STOP -> "MSG_STOP";
+                case MSG_INIT -> "MSG_INIT";
+                case MSG_STATE_COPY -> "MSG_STATE_COPY";
+                case MSG_COPY -> "MSG_COPY";
+                case MSG_ACCESS -> "MSG_ACCESS";
+                case MSG_NOTIFY_MAPMEMBER -> "MSG_NOTIFY_MAPMEMBER";
+                case MSG_PING -> "MSG_PING";
+                default -> "UNKNOWN";
+            };
         }
 
         public MapMessage(byte[] mapId, int msgtype, boolean diff, Serializable key, Serializable value,
