@@ -159,7 +159,7 @@ public class SoftReferenceObjectPool<T> extends BaseObjectPool<T> {
      * raises an exception on activation or fails validation,
      * {@code NoSuchElementException} is thrown. Exceptions thrown by
      * {@code MakeObject} are propagated to the caller; but other than
-     * {@code ThreadDeath} or {@code VirtualMachineError}, exceptions
+     * {@code VirtualMachineError}, exceptions
      * generated by activation, validation or destroy methods are swallowed
      * silently.
      * </p>
@@ -359,7 +359,7 @@ public class SoftReferenceObjectPool<T> extends BaseObjectPool<T> {
         PooledSoftReference<T> ref;
         while (iterator.hasNext()) {
             ref = iterator.next();
-            if (ref.getReference() == null || ref.getReference().isEnqueued()) {
+            if (ref.getReference() == null || ref.getReference().refersTo(null)) {
                 iterator.remove();
             }
         }
