@@ -37,9 +37,11 @@ import org.junit.Before;
 import org.junit.Test;
 
 import org.apache.catalina.Context;
+import org.apache.catalina.Globals;
 import org.apache.catalina.connector.Connector;
 import org.apache.catalina.startup.Tomcat;
 import org.apache.catalina.startup.TomcatBaseTest;
+import org.apache.tomcat.util.http.Method;
 import org.apache.tomcat.util.res.StringManager;
 
 public class TestAbstractAjpProcessor extends TomcatBaseTest {
@@ -90,7 +92,7 @@ public class TestAbstractAjpProcessor extends TomcatBaseTest {
         Assert.assertTrue(tomcat.getConnector().setProperty("packetSize", Integer.toString(ajpPacketSize)));
 
         // No file system docBase required
-        Context ctx = tomcat.addContext("", null);
+        Context ctx = getProgrammaticRootContext();
 
         Tomcat.addServlet(ctx, "snoop", new SnoopServlet());
         ctx.addServletMappingDecoded("/", "snoop");
@@ -389,7 +391,7 @@ public class TestAbstractAjpProcessor extends TomcatBaseTest {
     @Test
     public void testMethod() throws Exception {
         RequestDescriptor desc = new RequestDescriptor();
-        desc.putRequestInfo("REQUEST-METHOD", "LOCK");
+        desc.putRequestInfo("REQUEST-METHOD", Method.LOCK);
         desc.putRequestInfo("REQUEST-URI", "/testMethod");
         doSnoopTest(desc);
     }
@@ -466,7 +468,7 @@ public class TestAbstractAjpProcessor extends TomcatBaseTest {
         desc.putRequestInfo("REQUEST-REMOTE-HOST", "MYCLIENT");
         desc.putRequestInfo("REQUEST-REMOTE-ADDR", "10.1.2.3");
         desc.putRequestInfo("REQUEST-REMOTE-PORT", "34567");
-        desc.putRequestInfo("REQUEST-METHOD", "LOCK");
+        desc.putRequestInfo("REQUEST-METHOD", Method.LOCK);
         desc.putRequestInfo("REQUEST-URI", "/a/b/c");
         desc.putRequestInfo("REQUEST-PROTOCOL", "HTTP/1.x");
         desc.putRequestInfo("REQUEST-IS-SECURE", "true");
@@ -483,7 +485,7 @@ public class TestAbstractAjpProcessor extends TomcatBaseTest {
     @Test
     public void testSmallBody() throws Exception {
         RequestDescriptor desc = new RequestDescriptor();
-        desc.putRequestInfo("REQUEST-METHOD", "PUT");
+        desc.putRequestInfo("REQUEST-METHOD", Method.PUT);
         desc.putRequestInfo("REQUEST-CONTENT-LENGTH", "100");
         desc.putRequestInfo("REQUEST-BODY-SIZE", "100");
         desc.putRequestInfo("REQUEST-URI", "/testSmallBody");
@@ -493,7 +495,7 @@ public class TestAbstractAjpProcessor extends TomcatBaseTest {
     @Test
     public void testLargeBody() throws Exception {
         RequestDescriptor desc = new RequestDescriptor();
-        desc.putRequestInfo("REQUEST-METHOD", "PUT");
+        desc.putRequestInfo("REQUEST-METHOD", Method.PUT);
         desc.putRequestInfo("REQUEST-CONTENT-LENGTH", "10000");
         desc.putRequestInfo("REQUEST-BODY-SIZE", "10000");
         desc.putRequestInfo("REQUEST-URI", "/testLargeBody");
@@ -507,7 +509,7 @@ public class TestAbstractAjpProcessor extends TomcatBaseTest {
         tomcat.start();
 
         // No file system docBase required
-        Context ctx = tomcat.addContext("", null);
+        Context ctx = getProgrammaticRootContext();
 
         Tomcat.addServlet(ctx, "helloWorld", new HelloWorldServlet());
         ctx.addServletMappingDecoded("/", "helloWorld");
@@ -571,7 +573,7 @@ public class TestAbstractAjpProcessor extends TomcatBaseTest {
         tomcat.start();
 
         // No file system docBase required
-        Context ctx = tomcat.addContext("", null);
+        Context ctx = getProgrammaticRootContext();
 
         Tomcat.addServlet(ctx, "helloWorld", new HelloWorldServlet());
         ctx.addServletMappingDecoded("/", "helloWorld");
@@ -632,13 +634,13 @@ public class TestAbstractAjpProcessor extends TomcatBaseTest {
         validateCpong(ajpClient.cping());
 
         ajpClient.setUri("/test/echo-params.jsp");
-        ajpClient.setMethod("POST");
+        ajpClient.setMethod(Method.POST);
         TesterAjpMessage forwardMessage = ajpClient.createForwardMessage();
         forwardMessage.addHeader(0xA008, "9");
         if (multipleCL) {
             forwardMessage.addHeader(0xA008, "99");
         }
-        forwardMessage.addHeader(0xA007, "application/x-www-form-urlencoded");
+        forwardMessage.addHeader(0xA007, Globals.CONTENT_TYPE_FORM_URL_ENCODING);
         forwardMessage.end();
 
         TesterAjpMessage bodyMessage = ajpClient.createBodyMessage("test=data".getBytes());
@@ -678,7 +680,7 @@ public class TestAbstractAjpProcessor extends TomcatBaseTest {
         Tomcat tomcat = getTomcatInstance();
 
         // No file system docBase required
-        Context ctx = tomcat.addContext("", null);
+        Context ctx = getProgrammaticRootContext();
 
         Tomcat.addServlet(ctx, "bug55453", new Tester304WithBodyServlet());
         ctx.addServletMappingDecoded("/", "bug55453");
@@ -709,22 +711,22 @@ public class TestAbstractAjpProcessor extends TomcatBaseTest {
 
     @Test
     public void testZeroLengthRequestBodyGetA() throws Exception {
-        doTestZeroLengthRequestBody("GET", true);
+        doTestZeroLengthRequestBody(Method.GET, true);
     }
 
     @Test
     public void testZeroLengthRequestBodyGetB() throws Exception {
-        doTestZeroLengthRequestBody("GET", false);
+        doTestZeroLengthRequestBody(Method.GET, false);
     }
 
     @Test
     public void testZeroLengthRequestBodyPostA() throws Exception {
-        doTestZeroLengthRequestBody("POST", true);
+        doTestZeroLengthRequestBody(Method.POST, true);
     }
 
     @Test
     public void testZeroLengthRequestBodyPostB() throws Exception {
-        doTestZeroLengthRequestBody("POST", false);
+        doTestZeroLengthRequestBody(Method.POST, false);
     }
 
     private void doTestZeroLengthRequestBody(String method, boolean callAvailable) throws Exception {
@@ -732,7 +734,7 @@ public class TestAbstractAjpProcessor extends TomcatBaseTest {
         Tomcat tomcat = getTomcatInstance();
 
         // No file system docBase required
-        Context ctx = tomcat.addContext("", null);
+        Context ctx = getProgrammaticRootContext();
 
         ReadBodyServlet servlet = new ReadBodyServlet(callAvailable);
         Tomcat.addServlet(ctx, "ReadBody", servlet);
@@ -787,7 +789,7 @@ public class TestAbstractAjpProcessor extends TomcatBaseTest {
         Assert.assertTrue(tomcat.getConnector().setProperty("packetSize", Integer.toString(ajpPacketSize)));
 
         // No file system docBase required
-        Context ctx = tomcat.addContext("", null);
+        Context ctx = getProgrammaticRootContext();
 
         FixedResponseSizeServlet servlet = new FixedResponseSizeServlet(15000, 16000);
         Tomcat.addServlet(ctx, "FixedResponseSizeServlet", servlet);
@@ -829,7 +831,7 @@ public class TestAbstractAjpProcessor extends TomcatBaseTest {
         Tomcat tomcat = getTomcatInstance();
 
         // No file system docBase required
-        Context ctx = tomcat.addContext("", null);
+        Context ctx = getProgrammaticRootContext();
 
         Tomcat.addServlet(ctx, "bug66512", new InvalidHeaderServlet());
         ctx.addServletMappingDecoded("/", "bug66512");
@@ -883,6 +885,60 @@ public class TestAbstractAjpProcessor extends TomcatBaseTest {
     }
 
 
+    /*
+     * https://bz.apache.org/bugzilla/show_bug.cgi?id=66591
+     */
+    @Test
+    public void testNoHeaders() throws Exception {
+
+        Tomcat tomcat = getTomcatInstance();
+
+        // No file system docBase required
+        Context ctx = getProgrammaticRootContext();
+
+        Tomcat.addServlet(ctx, "bug66591", new NoHeadersServlet());
+        ctx.addServletMappingDecoded("/", "bug66591");
+
+        tomcat.start();
+
+        SimpleAjpClient ajpClient = new SimpleAjpClient();
+        ajpClient.setPort(getPort());
+        ajpClient.connect();
+
+        validateCpong(ajpClient.cping());
+
+        TesterAjpMessage forwardMessage = ajpClient.createForwardMessage();
+        forwardMessage.end();
+
+        TesterAjpMessage responseHeaderMessage = ajpClient.sendMessage(forwardMessage, null);
+
+        // Expect 3 messages: headers, body chunk, end
+        Map<String, List<String>> responseHeaders = validateResponseHeaders(responseHeaderMessage, 200, "200");
+        Assert.assertTrue(responseHeaders.isEmpty());
+
+        String body = extractResponseBody(ajpClient.readMessage());
+        Assert.assertTrue(body.isEmpty());
+
+        validateResponseEnd(ajpClient.readMessage(), true);
+
+        // Double check the connection is still open
+        validateCpong(ajpClient.cping());
+
+        ajpClient.disconnect();
+    }
+
+
+    private static class NoHeadersServlet extends HttpServlet {
+
+        private static final long serialVersionUID = 1L;
+
+        @Override
+        protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
+            resp.flushBuffer();
+        }
+    }
+
+
     /**
      * Process response header packet and checks the status. Any other data is ignored.
      */
@@ -943,7 +999,7 @@ public class TestAbstractAjpProcessor extends TomcatBaseTest {
         Assert.assertEquals(0x03, message.readByte());
 
         int len = message.readInt();
-        Assert.assertTrue(len > 0);
+        Assert.assertTrue(len >= 0);
         return message.readString(len);
     }
 
@@ -1052,7 +1108,7 @@ public class TestAbstractAjpProcessor extends TomcatBaseTest {
             response.setCharacterEncoding("UTF-8");
 
             try (PrintWriter w = response.getWriter()) {
-                w.println("Method: " + (isPost ? "POST" : "GET") + ". Reading request body...");
+                w.println("Method: " + (isPost ? Method.POST : Method.GET) + ". Reading request body...");
                 w.println("Request Body length in bytes: " + readCount);
             }
         }
